<?php

// Exit if accessed directly
defined( 'ABSPATH' ) || exit;

class WCSS_Share_Cart {

	public function __construct() {
		add_action( 'wp_ajax_wcss_share_medium_ajax', array( $this, 'share_medium_ajax' ) );
		add_action( 'wp_ajax_nopriv_wcss_share_medium_ajax', array( $this, 'share_medium_ajax' ) );
		add_action( 'wcss_before_share_medium_ajax', array( $this, 'empty_share_cart_view' ) );
		add_action( 'init', array( $this, 'delete_old_shared_cart_periodically' ) );
	}

	/**
	 * Get the uniqid 5 characters everytime.
	 *
	 * @return String
	 * @since 1.0
	 */
	public function get_uniqid() {

		$charset = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
		$base    = strlen( $charset );
		$result  = '';

		$now = explode( ' ', microtime() )[1];

		while ( $now >= $base ) {
			$i      = $now % $base;
			$result = $charset[ $i ] . $result;
			$now   /= $base;
		}

		return substr( $result, -5 );

	}

	/**
	 * Get current user cart session
	 *
	 * @return array     Return array of the current user cart session.
	 * @since 1.0
	 */
	public function get_cart_session() {

		$cart_session = array();

		$cart_session['cart']            = WC()->session->cart;
		$cart_session['cart_totals']     = WC()->session->cart_totals;
		$cart_session['applied_coupons'] = WC()->session->applied_coupons;

		return apply_filters( 'wcss_get_cart_session', $cart_session, WC() );
	}

	public function share_medium_ajax() {
		$cart_session = $this->get_cart_session();

		$uid = apply_filters( 'wcss_unique_cart_key', $this->get_uniqid() );

		do_action( 'wcss_before_share_medium_ajax', $uid, $cart_session );

		$shared_cart = wcss_insert_shared_cart( array(
			'cart_key'   => $uid,
			'cart_value' => $cart_session,
		) );

		$social_share_msg = esc_html( get_option( 'wcss_social_share_message' ) );
		$cart_share_link  = wcss_get_cart_link( $uid );

		// Replace {link} with actual link.
		$complied_msg = str_replace( '{link}', $cart_share_link, $social_share_msg );

		$messenger_app_id = wcss_share_integration_option( 'messenger', 'app_id' );

		if ( wp_is_mobile() ) {
			$whatsapp_link  = "https://api.whatsapp.com/send?text={$complied_msg}";
			$messenger_link = "fb-messenger://share/?link={$cart_share_link}&app_id={$messenger_app_id}";
		} else {
			$whatsapp_link  = "https://web.whatsapp.com/send?text={$complied_msg}";
			$messenger_link = "https://www.facebook.com/dialog/send?app_id={$messenger_app_id}&display=page&link=" . urlencode( $cart_share_link ) . '&redirect_uri=' . urlencode( home_url() );
		}

		wcss_get_template(
			'share-medium.php',
			apply_filters(
				'wcss_share_mediums',
				array(
					'uid'              => $uid,
					'cart_share_link'  => $cart_share_link,
					'print_cart_key'   => $uid,
					'whatsapp_link'    => $whatsapp_link,
					'email_link'       => "mailto:?body={$complied_msg}",
					'facebook_link'    => 'https://www.facebook.com/sharer.php?u=' . urlencode( $cart_share_link ) . "&quote={$complied_msg}",
					'twitter_link'     => "https://twitter.com/intent/tweet?url={$complied_msg}",
					'google_plus_link' => "https://plus.google.com/share?url={$complied_msg}",
					'skype_link'       => "https://web.skype.com/share?url={$complied_msg}",
					'messenger_link'   => $messenger_link,
				)
			)
		);

		do_action( 'wcss_after_share_medium_ajax', $uid, $cart_session );
		wp_die();
	}

	public function empty_share_cart_view() {
		// If cart is empty.
		if ( wcss_is_cart_empty() !== true ) {
			return;
		}

		wcss_get_template( 'empty-cart-message.php' );
		wp_die();
	}

	/**
	 * Delete old shared cart periodically.
	 *
	 * @since 2.0.1
	 *
	 * @return void
	 */
	public function delete_old_shared_cart_periodically() {
		global $wpdb;

		if ( ! get_transient( 'wcss_delete_old_shared_cart_periodically' ) ) {

			$period = get_option( 'wcss_automatically_delete_old_shared_cart_period' );

			/**
			 * Filter delete old shared cart period.
			 *
			 * @since 2.0.1
			 *
			 * @param int $period Period in days.
			 */
			$period = apply_filters( 'wcss_automatically_delete_old_shared_cart_period', $period );

			if ( $period && 'none' !== $period ) {


				$old_shared_carts = $wpdb->get_results(
					$wpdb->prepare(
						"SELECT cart_key
						FROM {$wpdb->prefix}wcss_shared_cart
						WHERE cart_created < DATE_SUB( NOW(), INTERVAL %d DAY )
						LIMIT 1000",
						$period
					)
				);

				if ( $old_shared_carts ) {
					foreach ( $old_shared_carts as $old_shared_cart ) {
						wcss_delete_shared_cart( $old_shared_cart->cart_key );
					}
				}
			}

			set_transient( 'wcss_delete_old_shared_cart_periodically', current_time( 'U' ), DAY_IN_SECONDS );
		}
	}

} // end of WCSS_Share_Cart

$wcss_share_cart = new WCSS_Share_Cart();
