<?php

// Exit if accessed directly
defined( 'ABSPATH' ) || exit;

/**
 * Get shared cart object.
 *
 * @since 2.0.0
 *
 * @param string $cart_key Optional. Shared cart key.
 * @return WCSS_Shared_Cart Shared cart object.
 */
function wcss_get_shared_cart( $cart = '' ) {
	return new WCSS_Shared_Cart( $cart );
}

/**
 * Get all shared carts.
 *
 * @since 2.0.0
 *
 * @return array|bool Array of shared cart objects. false otherwise.
 */
function wcss_get_shared_carts() {
	global $wpdb;

	$shared_carts  = array();
	$_shared_carts = $wpdb->get_results(
		"SELECT
			cart_key
		FROM
			{$wpdb->prefix}wcss_shared_cart
		WHERE cart_key NOT IN (
			SELECT cart_key FROM {$wpdb->prefix}wcss_saved_cart
		)
		ORDER BY id DESC"
	);

	if ( ! $_shared_carts ) {
		return false;
	}

	foreach ( $_shared_carts as $_shared_cart ) {
		$shared_carts[] = wcss_get_shared_cart( $_shared_cart->cart_key );
	}

	return $shared_carts;
}

/**
 * Get shared carts by fields.
 *
 * @since 2.0.0
 *
 * @param string $field Shared cart field. Like `cart_key`, `cart_shared_by`, `cart_retrieved`.
 * @param string $value Shared cart field value. Like Shared cart key, Shared cart by, Shared cart retrieved count.
 * @param string $order Optional. ASC or DESC. Default `DESC`.
 * @return array|bool Array of shared cart objects or false if no shared cart found.
 */
function wcss_get_shared_carts_by( $field, $value, $order = 'DESC' ) {
	if ( ! $field || ! $value ) {
		return false;
	}

	if ( 'user_id' !== $field && 'cart_key' !== $field && 'cart_retrieved' !== $field) {
		return false;
	}

	global $wpdb;

	$shared_carts  = array();
	$_shared_carts = $wpdb->get_results(
		$wpdb->prepare(
			"SELECT cart_key FROM {$wpdb->prefix}wcss_shared_cart WHERE {$field} = %s ORDER BY id {$order}",
			$value
		)
	);

	if ( ! $_shared_carts ) {
		return false;
	}

	foreach ( $_shared_carts as $_shared_cart ) {
		$shared_carts[] = wcss_get_shared_cart( $_shared_cart->cart_key );
	}

	return $shared_carts;
}

/**
 * Delete all shared carts.
 *
 * @since 2.0.0
 *
 * @return void
 */
function wcss_delete_shared_carts() {
	$shared_carts = wcss_get_shared_carts();

	if ( ! $shared_carts ) {
		return;
	}

	foreach ( $shared_carts as $shared_cart ) {
		$saved_cart = wcss_get_saved_cart( $shared_cart->get_cart_key() );
		$saved_cart->delete();
		$shared_cart->delete();
	}
}

/**
 * Delete shared cart.
 *
 * @since 2.0.0
 *
 * @param string $cart_key Shared cart key.
 * @return bool True if shared cart deleted. False otherwise.
 */
function wcss_delete_shared_cart( $cart_key ) {
	$shared_cart = wcss_get_shared_cart( $cart_key );
	$saved_cart  = wcss_get_saved_cart( $cart_key );

	if ( ! $shared_cart->is_exists() || $saved_cart->is_exists() ) {
		return false;
	}

	$shared_cart->delete();

	return true;
}

/**
 * Get shared cart data by DB column.
 *
 * @param string $cart_key Cart key.
 * @param string $column  Column name.
 * @return mixed|bool Returns data if found, false otherwise.
 */
function wcss_get_shared_cart_data( $cart_key, $column ) {
	$shared_cart = wcss_get_shared_cart( $cart_key );

	if ( 'cart_key' === $column ) {
		return $shared_cart->get_cart_key();
	} elseif ( 'cart_value' === $column ) {
		return $shared_cart->get_cart_value();
	} elseif ( 'cart_shared_by' === $column ) {
		return $shared_cart->get_cart_shared_by();
	} elseif ( 'cart_retrieved' === $column ) {
		return $shared_cart->get_cart_retrieved();
	} elseif ( 'cart_retrieved_time' === $column ) {
		return $shared_cart->get_cart_retrieved_time();
	} elseif ( 'cart_created' === $column ) {
		return $shared_cart->get_cart_created();
	}

	return false;
}

/**
 * Insert data into shared cart table.
 *
 * @since 2.0.0
 *
 * @param string $args Optional. Args used to create shared cart.
 * @return WCSS_Shared_Cart|bool Shared cart object on success, false otherwise.
 */
function wcss_insert_shared_cart( $args = '' ) {
	$a = wp_parse_args( $args, array(
		'cart_key'            => '',
		'cart_value'          => '',
		'cart_shared_by'      => 0,
		'cart_retrieved'      => 0,
		'cart_retrieved_time' => null,
		'cart_created'        => '',
	) );

	$shared_cart = wcss_get_shared_cart();

	if ( $shared_cart->is_exists() ) {
		return false;
	}

	$shared_cart->set_cart_key( $a['cart_key'] );
	$shared_cart->set_cart_value( $a['cart_value'] );
	$shared_cart->set_cart_shared_by( $a['cart_shared_by'] );
	$shared_cart->set_cart_retrieved( $a['cart_retrieved'] );
	$shared_cart->set_cart_retrieved_time( $a['cart_retrieved_time'] );
	$shared_cart->set_cart_created( $a['cart_created'] );

	$inserted_shared_cart = $shared_cart->create();

	if ( ! $inserted_shared_cart ) {
		return false;
	}

	return $inserted_shared_cart;
}

/**
 * Increment the saved cart retrieved count value by one.
 *
 * @since 2.0.0
 *
 * @param string $cart_key Cart key.
 * @return bool True on success, false otherwise.
 */
function wcss_shared_cart_retrieved_count_increment( $cart_key ) {
	$shared_cart = wcss_get_shared_cart( $cart_key );

	if ( ! $shared_cart->is_exists() ) {
		return false;
	}

	$shared_cart->set_cart_retrieved( $shared_cart->get_cart_retrieved() + 1 );

	return $shared_cart->update();
}

/**
 * Set cart retrieved time.
 *
 * @param string $cart_key Cart key.
 * @return bool True on success, false otherwise.
 */
function wcss_set_shared_cart_retrieved_time( $cart_key ) {
	$shared_cart = wcss_get_shared_cart( $cart_key );

	if ( ! $shared_cart->is_exists() ) {
		return false;
	}

	$shared_cart->set_cart_retrieved_time( current_time( 'mysql' ) );

	return $shared_cart->update();
}

/**
 * Get shared cart retrieved time.
 *
 * @since 2.0.0
 *
 * @param string $cart_key Cart key.
 * @return string|bool Returns cart retrieved time on success, false otherwise.
 */
function wcss_get_shared_cart_retrieved_time( $cart_key ) {
	$shared_cart = wcss_get_shared_cart( $cart_key );

	if ( ! $shared_cart->is_exists() ) {
		return false;
	}

	return $shared_cart->get_cart_retrieved_time();
}

/**
 * Get the shared cart retrieved count.
 *
 * @since 2.0.0
 *
 * @param string $cart_key Cart key.
 * @return int Returns cart retrieved count on success, false otherwise.
 */
function wcss_get_shared_cart_retrieved_count( $cart_key ) {
	$shared_cart = wcss_get_shared_cart( $cart_key );

	if ( ! $shared_cart->is_exists() ) {
		return false;
	}

	return $shared_cart->get_cart_retrieved();
}

/**
 * Get shared cart session.
 *
 * @param string $cart_key Cart key.
 * @return array Returns cart session on success, false otherwise.
 */
function wcss_get_shared_cart_session( $cart_key ) {
	$shared_cart = wcss_get_shared_cart( $cart_key );

	if ( ! $shared_cart->is_exists() ) {
		return false;
	}

	return $shared_cart->get_cart_value();
}
